-- buEdgeSketch.mcr Ver.1.5b
-- kazuhiro kasiwagi (kasiwagi@ummb.jp)
-------------------------------------------------------------------------------------------------
-- select vert			shift + over the vert
-- create edge			drag on face
-- remove edge			drag on edge
-- spin edge			shift + drag on edge
-- cut&connect			alt + drag on edge (+ctrl:middle cut)
-- collapse edge		alt + shift + drag on edge
-------------------------------------------------------------------------------------------------
-- Ver 1.5 cuttool rewrite
-- Ver 1.2 next beta
-------------------------------------------------------------------------------------------------
-- Thanks > Rene.Baca

macroScript buEdgeSketch
	category:"Bu"
	buttontext:"buEdgeSketch"
	toolTip:"buEdgeSketch" 
(
	local targetObj
	local loopF
	local ms
    local samplepoint = #([0,0],[-5,5],[10,0],[0,-10],[0,15],[-15,0],[20,0],[0,-25],[-10,25],[20,25],[-25,-25])

	fn getPolyFace i = (
		local v = meshop.getVertsUsingFace ms i as array
		local f1 = polyOp.getFacesUsingVert targetObj v[1]
		local f2 = polyOp.getFacesUsingVert targetObj v[2]
		local f3 = polyOp.getFacesUsingVert targetObj v[3]
		return ((f1*f2*f3) as array)[1]
	)

	fn getNearPoint i p = (
		local v = meshop.getVertsUsingFace ms i as array
		d = for i in v collect length (p - (meshop.getVert ms i))
		return #{v[(findItem d (amin d))]}
	)

	fn createEdge f vp1 vp2 = (
		local vi = polyOp.getVertsUsingFace targetObj f as array
		local l1Ar = #(),l2Ar = #()
		for i in vi do (
			local tmp = polyOp.getVert targetObj i - vp1
			append l1Ar (length tmp)
			append l2Ar (dot (normalize tmp) (normalize (vp2 - vp1)))
		)
		targetObj.createEdge vi[(findItem l1Ar (amin l1Ar))] vi[(findItem l2Ar (amax l2Ar))]
	)

	fn collapseEdge p1 p2 = (
		local e = (polyOp.getEdgesUsingFace targetObj p1) * (polyOp.getEdgesUsingFace targetObj p2)
		if e.numberSet >= 1 do (
			polyOp.setEdgeSelection targetObj e
			targetObj.collapse #edge
		)
	)

	fn deleteEdge p1 p2 = (
		local e = (polyOp.getEdgesUsingFace targetObj p1) * (polyOp.getEdgesUsingFace targetObj p2)
		if e.numberSet >= 1 do (
			polyOp.setEdgeSelection targetObj e
			targetObj.remove selLevel:#edge
		)
	)

	fn spinEdge p1 p2 = (
		local e = (polyOp.getEdgesUsingFace targetObj p1) * (polyOp.getEdgesUsingFace targetObj p2)
		local v1 = polyOp.getVertsUsingFace targetObj p1
		local v2 = polyOp.getVertsUsingFace targetObj p2
		if v1.numberSet == 3 and v2.numberSet == 3 and e.numberSet == 1 do -- tri only
		(
			local v12 = polyOp.getVertsUsingEdge targetObj e
			v1 = ((v1 - v12) as array)[1]
			v2 = ((v2 - v12) as array)[1]
			polyOp.setEdgeSelection targetObj e
			targetObj.remove selLevel:#edge -- smart remove with ctrl
			targetObj.createEdge v1 v2
		)
	)

	fn cutEdge vp1 vp2 v3 e = (
		local v12ar = polyOp.getVertsUsingEdge targetObj #{e} as array
		local v1 = polyOp.getVert targetObj v12ar[1]
		local v2 = polyOp.getVert targetObj v12ar[2]
		local mpl = manip.makePlaneFromNormal (cross (vp2 - vp1) v3) vp1
		local v = [0,0,0]
		mpl.intersect (ray v1 (v2 - v1)) &v
		
		local targetV = targetObj.insertVertexInEdge e 0.5
		if not keyboard.controlPressed do polyop.setVert targetObj targetV v -- by Rene.Baca
		local vsel = polyOp.getVertSelection targetObj
		if vsel.numberSet == 1 do (
			vsel += #{targetV}
			polyOp.setVertSelection targetObj vsel
			targetObj.ConnectVertices()
		)
		polyOp.setVertSelection targetObj targetV
		redrawViews()
	)

	fn checkAcrossEdge f p1 p2 = (
		local e = polyOp.getEdgesUsingFace targetObj f
		for i in e do (
			local v12ar = polyOp.getVertsUsingEdge targetObj i as array
			local v1 = polyOp.getVert targetObj v12ar[1]
			local v2 = polyOp.getVert targetObj v12ar[2]
			
			if	(dot (cross (v2-v1) (p1-v1)) (cross (v2-v1) (p2-v1))) < 0 and 
				(dot (cross (p2-p1) (v1-p1)) (cross (p2-p1) (v2-p1))) < 0 then return i
		)
		return false
	)

	fn checkStroke mR1 mR2 = (
		local mf1 = intersectRayEx ms mR1
		local mf2 = intersectRayEx ms mR2

		if not keyboard.shiftPressed and keyboard.altPressed then (
			if mf1 == undefined then (
				swap mf1 mf2
				swap mR1 mR2
			)
			if mf1 != undefined then (
				local mpl = manip.makePlaneFromNormal mf1[1].dir mf1[1].pos
				local v = [0,0,0]
				mpl.intersect mR2 &v
				local e = checkAcrossEdge (getPolyFace mf1[2]) mf1[1].pos v
				if e != false then cutEdge mf1[1].pos v mR1.dir e
			)
			return()
		)
		if (mf1 != undefined) and (mf2 != undefined) do (
			local pf1 = getPolyFace mf1[2]
			local pf2 = getPolyFace mf2[2]
			if pf1 == pf2 then createEdge pf1 mf1[1].pos mf2[1].pos
			else if keyboard.shiftPressed and keyboard.altPressed then collapseEdge pf1 pf2
			else if keyboard.shiftPressed then spinEdge pf1 pf2
			else deleteEdge pf1 pf2
		)
	)

	tool mtEdgeSketch
	(
		local mR1,mR2
		on freeMove do (
			if keyboard.shiftPressed do
				try (
                    local mf1 = undefined
                    for p in samplepoint while mf1 == undefined do (
                        mR1 = mapScreenToWorldRay (viewPoint + p)
                        mf1 = intersectRayEx ms mR1
                    )
					if mf1 != undefined then
						polyOp.setVertSelection targetObj (getNearPoint mf1[2] mf1[1].pos)
					else
						polyOp.setVertSelection targetObj #{}
					redrawViews()
				) catch ()
		)
		
		on mousePoint clickno do (
			if clickno == 1 then
				mR1 = mapScreenToWorldRay viewPoint
			else (
				mR2 = mapScreenToWorldRay viewPoint
				checkStroke mR1 mR2
				#stop
			)
		)
		on mouseAbort clickno do loopF = false
	)

	On IsEnabled Return Filters.Is_EPoly()
	On IsVisible Return Filters.Is_EPoly()
	On Execute Do
	(
		enableUndo off -- mousetool cannot undo
		loopF = true
		try (
			if (targetObj = selection[1]) != undefined and (classof targetObj) == Editable_Poly do
				while loopF do (
					ms = snapshot targetObj
					ms.isHidden = true
					startTool mtEdgeSketch
					delete ms
				)
		) catch ()
		enableUndo on
	)
)
