-- buNudge.ms Ver.1.5
-- Kazuhiro Kasiwagi (kasiwagi@ummb.jp)
-- required Avguard Extensions

struct CbuNudge
(
	mode,dec,obj,
	ANGLERANGE = -0.9,
	selVert = #{},

	fn getMoveVal = (
		local scSize = getViewSize()
		minP = mapScreenToView [0,0] 0
		maxP = mapScreenToView scSize 0
		case mode of (
			1: -((Inverse(getViewTM())).row1)*(maxP.x - minP.x)/scSize.x*dec
			2: ((Inverse(getViewTM())).row1)*(maxP.x - minP.x)/scSize.x*dec
			3: -((Inverse(getViewTM())).row2)*(maxP.y - minP.y)/scSize.y*dec
			4: ((Inverse(getViewTM())).row2)*(maxP.y - minP.y)/scSize.y*dec
		)
	),
	
	fn moveEP = ( -- editable poly
		local sV = 	case subObjectLevel of (
						0: undefined
						1: polyop.getVertSelection obj
						2: polyop.getVertsUsingEdge obj #selection
						3: polyop.getVertsUsingEdge obj #selection
						4: polyop.getVertsUsingFace obj #selection
						5: polyop.getVertsUsingFace obj #selection
					)
		local v = getMoveVal()
		if polyOp.getVDataChannelSupport obj 1 then ( -- soft sel
			local b = copy -sV
			for i in b where (polyOp.getVDataValue obj 1 i > 0) do sV[i] = true
		)
		local ar = for i=1 to sV.numberSet collect v

		polyop.moveVert obj sV ar useSoftSel:true
	),
	
	fn moveUVW = ( -- UVW
		local mp = case mode of (
						1: [-1,0,0]
						2: [ 1,0,0]
						3: [0, 1,0]
						4: [0,-1,0]
					)
		
		obj.moveSelected (mp * dec / 512)
	),

	fn checkViewType = (
		case viewport.getType() of (
			#view_persp_user: return true
			#view_camera: return true
			#view_spot: return true
			#view_shape: return true
			default: return false
		)
	),
	
	fn getNextVert curVert nextVec addF= (
		local selEdge = (polyOp.getEdgesUsingVert obj curVert) as array
		local v1 = polyOp.getVert obj curVert
		local curVec = for i in selEdge collect ( -- check all edge vec
			local tmpVi = (((polyOp.getVertsUsingEdge obj i) - #{curVert}) as array)[1]
			normalize( (polyOp.getVert obj tmpVi) - v1)
		)
		local vAr = for i=1 to curVec.count collect dot curVec[i] nextVec
		local ei = findItem vAr (amax vAr) -- next edge index
		if vAr[ei] < ANGLERANGE then
			selVert += #{curVert}
		else
			selVert += (polyOp.getVertsUsingEdge obj selEdge[ei]) - (if addF then #{} else #{curVert})
	),

	fn moveMode m = -- mov vert
	(
		if 5 > m then -- set mode and dec(1px or 10px)
		(
			dec = 1.0
			mode = m
		) else (
			dec = 10.0
			mode = m - 4
		)

		if subObjectLevel == 0 then
		(
			move selection (getMoveVal())
		) else (
			setCommandPanelTaskMode #modify
			obj = modPanel.getCurrentObject()
			case (classof obj) of
			(
				Unwrap_UVW: moveUVW()
				Editable_Poly: moveEP()
			)
		)
	),
	
	fn selectModeV m = -- vert
	(
		local addF = false
		if 5 <= m do
		(
			m -= 4
			addF = True
		)
		
		selVert = #{}
		local selVertInit = polyOp.getVertSelection obj 
		for i in selVertInit do
		(
			local sf = polyop.getFacesUsingVert obj i
			local n = [0,0,0]
			for j in sf do
				n += polyOp.getFaceNormal obj j
			n = normalize n
			
			local nextVec = case m of
			(
				1: cross n ((Inverse(getViewTM())).row2)
				2: cross n -((Inverse(getViewTM())).row2)
				3: cross n ((Inverse(getViewTM())).row1)
				4: cross n -((Inverse(getViewTM())).row1)
			)
			getNextVert i nextVec addF
		)

		polyOp.setVertSelection obj selVert
		redrawViews()
	),

	fn selectModeE m = -- edge
	(
		local addF = true
		if 5 <= m do
		(
			m -= 4
			addF = False
		)
		case m of
		(
			1: obj.setRingShift 1 addF true
			2: obj.setRingShift -1 addF true
			3: obj.setLoopShift 1 addF true
			4: obj.setLoopShift -1 addF true
		)
	),

	fn selectMode m = ( -- mov sel
		case subobjectlevel of (
			1: selectModeV m
			2: selectModeE m
		)
	),

	fn skinGetSelVert = (
		num = skinOps.GetNumberVertices obj
		for i = 1 to num where (skinOps.isVertexSelected obj i) == 1 collect i
	),

	fn skinGetVWBone m = (
		sv = (skinGetSelVert())[1]
		if sv == undefined do return()
		bn = skinOps.GetVertexWeightCount obj sv
		bAr = for i=1 to bn collect skinOps.GetVertexWeightBoneID obj sv i
		idx = findItem bAr (skinOps.GetSelectedBone obj)
		if m then idx += 1 else idx -=1
		if idx == 0 do idx = bn
		if idx > bn do idx = 1
		skinOps.SelectBone obj bAr[idx]
	),

    fn skinSelvert m = (
        m -= 4
		base = selection[1]
        sv = for i = 1 to base.verts.count where (skinOps.IsVertexSelected obj i) == 1 collect i
		local addF = false
		
        sobj = obj
        obj = base.baseobject
		selVert = #{}
		local selVertInit = sv
		for i in selVertInit do (
			local sf = polyop.getFacesUsingVert obj i
			local n = [0,0,0]
			for j in sf do
				n += polyOp.getFaceNormal obj j
			n = normalize n
			
			local nextVec = case m of (
				1: cross n ((Inverse(getViewTM())).row2)
				2: cross n -((Inverse(getViewTM())).row2)
				3: cross n ((Inverse(getViewTM())).row1)
				4: cross n -((Inverse(getViewTM())).row1)
			)
			getNextVert i nextVec addF
		)

        skinOps.SelectVertices sobj selVert
		redrawViews()
    ),
	
	fn skinEditWait m = (
		case m of (
			1: skinOps.AddWeight obj -0.05
			2: skinOps.AddWeight obj 0.05
			3: skinGetVWBone true
			4: skinGetVWBone false
			-- 5: skinOps.SetWeight obj 0.0
			-- 6: skinOps.SetWeight obj 1.0
			-- 7: skinOps.SelectPreviousBone obj
			-- 8: skinOps.SelectNextBone obj
			5: skinSelvert m
			6: skinSelvert m
			7: skinSelvert m
			8: skinSelvert m
		)
	),

	fn run m = (
		obj = selection[1]
		if checkViewType() do return()

		setCommandPanelTaskMode #modify
		obj = modPanel.getCurrentObject()
		case (classof obj) of (
			skin: skinEditWait m
			default: 
				case toolmode.commandmode of (
					#move: moveMode m
					#select: selectMode m
				)
		)

	)
)
global buNudge = CbuNudge()

